package tech.deepq.source.spel;

import cn.hutool.core.collection.CollUtil;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import tech.deepq.common.db.entity.Store;
import tech.deepq.common.db.entity.vo.TaskMiniVO;
import tech.deepq.common.db.service.StoreManager;
import tech.deepq.common.dto.HttpRes;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.StringJoiner;

/**
 * @author yechuan
 * @since 2023/9/6 22:57
 **/
@Slf4j
@Component
public class ZhiDeMaiRequest {

    DateTimeFormatter timeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    @Autowired
    private StoreManager storeManager;

    private static boolean init = false;

    public long getLatestEventId(String initEventId) {

        if (!init && initEventId != null) {
            init = true;
            return Long.parseLong(initEventId);
        }

        Store store = storeManager.getValueFromNsByKey("zdm", "latestEventId");
        if (store == null || store.getDeleted().compareTo(1) == 0) {
            return 0L;
        }
        return Long.parseLong(store.getValue());
    }


    /**
     * 判断是否满足下一页event条件
     */
    public boolean hasNextEvent(HttpRes eventRes, JsonObject filterArgs) {

        return CollUtil.isNotEmpty(filterArgs.get("data").getAsJsonArray());
    }

    /**
     * 构建下一页event消息内容
     */
    public TaskMiniVO buildNextEventPayload(HttpRes eventRes, JsonObject filterArgs, TaskMiniVO taskMiniVO) {
        long maxEventId = Long.parseLong(eventRes.getRequestParams().getPlaceholderExpressionsValueCache().get("#{event_id}").toString());


        for (JsonElement eventObject : filterArgs.get("data").getAsJsonArray()) {
            JsonObject event = eventObject.getAsJsonObject();
            long eventId = event.get("event_id").getAsLong();
            maxEventId = Math.max(eventId, maxEventId);
        }
        if (taskMiniVO.getArgs() == null) {
            taskMiniVO.setArgs(new HashMap<>());
        }
        taskMiniVO.getArgs()
                .put("event_id", maxEventId + "");

        storeManager.upsert("zdm", "latestEventId", maxEventId + "");
        return taskMiniVO;
    }

    /**
     * 判断是否满足资讯下架事件
     */
    public boolean hasDelArticle(HttpRes eventRes, JsonObject filterArgs) {
        LocalDate today = LocalDate.now();


        for (JsonElement eventObject : filterArgs.get("data").getAsJsonArray()) {
            JsonObject event = eventObject.getAsJsonObject();
            String creationDate = event.get("creation_date").getAsString();
            if (!today.isEqual(LocalDateTime.parse(creationDate, timeFormatter).toLocalDate())) {
                continue;
            }

            String action = event.get("action").getAsString();
            if ("del".equals(action)) {
                return true;
            }
        }

        return false;
    }

    /**
     * 构建资讯下架消息内容
     */
    public String buildDelArticlePayload(HttpRes eventRes, JsonObject filterArgs, TaskMiniVO taskMiniVO) {
        LocalDate today = LocalDate.now();

        JsonObject result = new JsonObject();
        JsonArray delDelArticleArray = new JsonArray();
        result.add("data", delDelArticleArray);
        for (JsonElement eventObject : filterArgs.get("data").getAsJsonArray()) {
            JsonObject event = eventObject.getAsJsonObject();
            String creationDate = event.get("creation_date").getAsString();
            if (!today.isEqual(LocalDateTime.parse(creationDate, timeFormatter).toLocalDate())) {
                continue;
            }

            String action = event.get("action").getAsString();
            if ("del".equals(action)) {
                JsonObject article = new JsonObject();
                article.add("article_id", event.get("main_id"));
                article.add("update_time", event.get("creation_date"));
                article.add("action", event.get("action"));
                delDelArticleArray.add(article);
            }
        }
        return result.getAsString();
    }


    /**
     * 判断是否满足资讯更新事件
     */
    public boolean hasUpdateOrInsertArticle(HttpRes eventRes, JsonObject filterArgs) {
        LocalDate today = LocalDate.now();

        for (JsonElement eventObject : filterArgs.get("data").getAsJsonArray()) {
            JsonObject event = eventObject.getAsJsonObject();
            String creationDate = event.get("creation_date").getAsString();
            if (!today.isEqual(LocalDateTime.parse(creationDate, timeFormatter).toLocalDate())) {
                continue;
            }

            String action = event.get("action").getAsString();
            if ("upd".equals(action) || "pub".equals(action)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 构建资讯更新消息内容
     */
    public TaskMiniVO buildUpdateOrInsertArticlePayload(HttpRes eventRes, JsonObject filterArgs, TaskMiniVO taskMiniVO) {
        LocalDate today = LocalDate.now();
        if (taskMiniVO.getArgs() == null) {
            taskMiniVO.setArgs(new HashMap<>());
        }

        StringJoiner stringJoiner = new StringJoiner(",");

        for (JsonElement eventObject : filterArgs.get("data").getAsJsonArray()) {
            JsonObject event = eventObject.getAsJsonObject();
            String creationDate = event.get("creation_date").getAsString();
            if (!today.isEqual(LocalDateTime.parse(creationDate, timeFormatter).toLocalDate())) {
                continue;
            }

            String action = event.get("action").getAsString();
            if ("upd".equals(action) || "pub".equals(action)) {
                stringJoiner.add(event.get("main_id").getAsString());
            }
        }
        taskMiniVO.getArgs()
                .put("article_ids", stringJoiner.toString());
        return taskMiniVO;
    }
}
